@@ -6,7 +6,7 @@ from collections import namedtuple, deque, OrderedDict
 import casadi as ca
 import numpy as np
 import itertools
-from typing import Union, Dict
+from typing import Union, Dict, Iterable
 
 from pymoca import ast
 from pymoca.tree import TreeWalker, TreeListener, flatten
@@ -15,6 +15,8 @@ from .alias_relation import AliasRelation
 from .model import Model, Variable, DelayArgument
 from .mtensor import _MTensor, _new_mx
 
+from ._options import _merge_default_options
+
 logger = logging.getLogger("pymoca")
 
 # TODO
@@ -35,7 +37,9 @@ OP_MAP = {'*': "__mul__",
           '==': '__eq__',
           "min": "fmin",
           "max": "fmax",
-          "abs": "fabs"}
+          "abs": "fabs",
+          "and": "__mul__",
+          "or": "__add__"}
 
 ForLoopIndexedSymbol = namedtuple('ForLoopIndexedSymbol', ['tree', 'transpose', 'indices'])
 
@@ -79,6 +83,11 @@ class GeneratorWalker(TreeWalker):
             return True
         return skip
 
+    def order_keys(self, keys: Iterable[str]):
+        # Symbols must come before classes, as we need to access symbol values when creating
+        # CasADi interpolant functions.
+        return sorted(keys, key=lambda attr: 0 if attr == 'symbols' else 1)
+
 
 # noinspection PyPep8Naming,PyUnresolvedReferences
 class Generator(TreeListener):
@@ -93,12 +102,12 @@ class Generator(TreeListener):
         self.for_loops = deque()
         self.functions = {}
         self.entered_classes = deque()
-        self.map_mode = 'inline' if options.get('unroll_loops', True) else 'serial'
-        self.function_mode = (True, False) if options.get('inline_functions', True) else (False, True)
+        self.map_mode = 'inline' if options['unroll_loops'] else 'serial'
+        self.function_mode = (True, False) if options['inline_functions'] else (False, True)
         self.delay_counter = 0
 
         # NOTE: Part of MTensor workaround.
-        self._expand_vectors_enabled = options.get('expand_vectors', False)
+        self._expand_vectors_enabled = options['expand_vectors']
 
     @property
     def current_class(self):
@@ -120,11 +129,20 @@ class Generator(TreeListener):
                 for a in ast.Symbol.ATTRIBUTES:
                     v = self.get_mx(getattr(ast_symbol, a))
                     if v is not None:
-                        if isinstance(v, ca.DM) and modelica_shape == ((None,),):
+                        if isinstance(v, ca.DM) and all(x == (None,) for x in modelica_shape):
                             # Scalar numeric type that behaves like an array.
                             # Coerce to Pyhton type to avoid interpretation
                             # issues.
                             v = python_type(v)
+                        elif isinstance(v, (float, int)) and not isinstance(v, python_type):
+                            # We skip booleans for now, as users likely depend
+                            # on them being integer/float-like.
+                            try:
+                                v = python_type(v)
+                            except (OverflowError, ValueError):
+                                # Cannot convert NaN/infs to integer
+                                pass
+
                         setattr(variable, a, v)
                 variable.prefixes = ast_symbol.prefixes
             variables.append(variable)
@@ -211,6 +229,8 @@ class Generator(TreeListener):
             src = self.get_derivative(v)
         elif op == '-' and n_operands == 1:
             src = -self.get_mx(tree.operands[0])
+        elif op == 'not' and n_operands == 1:
+            src = ca.if_else(self.get_mx(tree.operands[0]), 0, 1, True)
         elif op == 'mtimes':
             assert n_operands >= 2
             src = self.get_mx(tree.operands[0])
@@ -286,7 +306,47 @@ class Generator(TreeListener):
 
             delay_argument = DelayArgument(expr, duration)
             self.model.delay_arguments.append(delay_argument)
-
+        elif op == '_pymoca_interp1d' and n_operands >= 3 and n_operands <= 4:
+            entered_class = self.entered_classes[-1]
+            if isinstance(tree.operands[0], ast.ComponentRef):
+                xp = self.get_mx(entered_class.symbols[tree.operands[0].name].value)
+            else:
+                xp = self.get_mx(tree.operands[0])
+            if isinstance(tree.operands[1], ast.ComponentRef):
+                yp = self.get_mx(entered_class.symbols[tree.operands[1].name].value)
+            else:
+                yp = self.get_mx(tree.operands[1])
+            arg = self.get_mx(tree.operands[2])
+            if n_operands == 4:
+                assert isinstance(tree.operands[3], ast.Primary)
+                mode = tree.operands[3].value
+            else:
+                mode = 'linear'
+            func = ca.interpolant('interpolant', mode, [xp], yp)
+            src = func(arg)
+        elif op == '_pymoca_interp2d' and n_operands >= 5 and n_operands <= 6:
+            entered_class = self.entered_classes[-1]
+            if isinstance(tree.operands[0], ast.ComponentRef):
+                xp = self.get_mx(entered_class.symbols[tree.operands[0].name].value)
+            else:
+                xp = self.get_mx(tree.operands[0])
+            if isinstance(tree.operands[1], ast.ComponentRef):
+                yp = self.get_mx(entered_class.symbols[tree.operands[1].name].value)
+            else:
+                yp = self.get_mx(tree.operands[1])
+            if isinstance(tree.operands[2], ast.ComponentRef):
+                zp = self.get_mx(entered_class.symbols[tree.operands[2].name].value)
+            else:
+                zp = self.get_mx(tree.operands[2])
+            arg_1 = self.get_mx(tree.operands[3])
+            arg_2 = self.get_mx(tree.operands[4])
+            if n_operands == 6:
+                assert isinstance(tree.operands[5], ast.Primary)
+                mode = tree.operands[5].value
+            else:
+                mode = 'linear'
+            func = ca.interpolant('interpolant', mode, [xp, yp], np.array(zp).ravel(order='F'))
+            src = func(ca.vertcat(arg_1, arg_2))
         elif op in OP_MAP and n_operands == 2:
             lhs = ca.MX(self.get_mx(tree.operands[0]))
             rhs = ca.MX(self.get_mx(tree.operands[1]))
@@ -685,7 +745,10 @@ class Generator(TreeListener):
             dep = s.dep()
             if dep.name() not in self.derivative:
                 der_dep = _new_mx("der({})".format(dep.name()), dep.size())
+                der_dep._modelica_shape = \
+                    self.nodes[self.current_class][dep.name()]._modelica_shape
                 self.derivative[dep.name()] = der_dep
+                self.nodes[self.current_class][der_dep.name()] = der_dep
                 return der_dep[slice_info['start']:slice_info['stop']:slice_info['step']]
             else:
                 return self.derivative[dep.name()][slice_info['start']:slice_info['stop']:slice_info['step']]
@@ -873,8 +936,9 @@ def generate(ast_tree: ast.Tree, model_name: str, options: Dict[str, bool]=None)
     :param options: dictionary of generator options
     :return: casadi model
     """
-    if options is None:
-        options = {}
+    options = _merge_default_options(options)
+
+
     component_ref = ast.ComponentRef.from_string(model_name)
     ast_walker = GeneratorWalker()
     flat_tree = flatten(ast_tree, component_ref)
